#
# Licensed Materials - Property of IBM
# 5655-FLW (C) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

import os.path, sys

from java.lang import Integer, System
from java.util import HashMap
# only classes from plugins/*runtime*.jar and plugins/*admin*.jar are visible to Jython in WAS 7
from com.ibm.bpe.database.migration import DataMigrationMgr

execfile(os.path.join(System.getProperty("was.install.root"), "ProcessChoreographer/config/bpclib.py"))
__file__ = "migrateDB.py"

#*******************************************************************************************************
# globals
#*******************************************************************************************************
MIN_SLICE = 10
MAX_SLICE = 50000
DEFAULT_SLICE = 50

#*******************************************************************************************************
# print usage and error message, if any
#*******************************************************************************************************
def usage(errmsg = None):
  if errmsg:
    print
    print "ERROR: " + errmsg
  #endif
  print
  print "Usage: wsadmin -conntype NONE -f " + __file__
  print "       (-server <serverName> | -cluster <clusterName>)"
  print "       [-dbUser <userID>] -dbPassword <password> [-dbSchema <schema> | -dbDesign <file>]"
  print "       [-slice <slice>] [-skipTablespaceMigration] [-force]"
  print
  print "Migrates the data in a pre-6.2 Business Process Choreographer database. This script must"
  print "be run in disconnected mode and the application server or cluster members must not be running."
  print
  print "Note that this script must be run on the node of the server (when '-server' was specified) or"
  print "on the node of a cluster member (when '-cluster' was specified). That is, it cannot be run on"
  print "the deployment manager node in a network deployment (ND) environment."
  print
  print "The '-server' and '-cluster' parameters denote the Business Process Choreographer"
  print "  configuration for which to migrate the database."
  print "The '-dbUser' and '-dbPassword' parameters are used to authenticate with the database."
  print "The optional '-dbSchema' parameter can be used to overwrite an implicit schema qualifier."
  print "  Do not use this parameter if an explicit schema qualifier has been configured."
  print "The optional '-dbDesign' parameter can be use to specify a database design file. Settings"
  print "  like the schema qualifier will be read from this file."
  print "The optional '-slice' parameter can be used to overwrite the default transaction slice size (" + str(DEFAULT_SLICE) + ")."
  print "  Its value must be between " + str(MIN_SLICE) + " and " + str(MAX_SLICE) + "."
  print "The optional '-skipTablespaceMigration' parameter allows to skip the tablespace migration for DB2"
  print "  if the tables are already contained in a suitable table space."
  print "The optional '-force' parameter might be needed if the script stopped before it completed the migration."
  print "  Using this option is the only way to run the script again, the script will attempt to continue migrating"
  print "  the data from the point that it had reached."
  print "  ATTENTION: Only use this parameter if you are sure that no other instance of the migration script is"
  print "  still running. If more than one instance runs at the same time, the database might become inconsistent,"
  print "  and require a restore from your backup."
#enddef

#*******************************************************************************************************
# main
#*******************************************************************************************************

# ----------------------------------------------------
# make sure the script is running in disconnected mode
if wsadmin.getConnectionStatus() == WsAdmin.CONNECTED:
  usage(__file__ + " cannot be run in connected mode, re-run using the '-conntype NONE' option.")

if wsadmin.getConnectionStatus() != WsAdmin.DISCONNECTED: # connected or error -> exit
  sys.exit(1)

# ----------------------
# parse the command line
props = {}
props.update({ "server": "", "cluster": "", "dbUser": "", "dbPassword": "", "dbSchema": "", "dbDesign": "" })
props.update({ "slice": DEFAULT_SLICE, "force": None, "skipTablespaceMigration": None })
try:
  props = wsadmin.parse(sys.argv, props)
  props["node"] = System.getProperty("local.node")
except ValueError, error:
  usage(str(error))
  sys.exit(1)
#endtry

# -----------------------------
# check command line parameters
if not props["server"] and not props["cluster"]:
  usage("One of the '-server' and '-cluster' parameters must be specified.")
  sys.exit(1)
#endif

if props["server"] and props["cluster"]:
  usage("Only one of the '-server' and '-cluster' parameters must be specified.")
  sys.exit(1)
#endif

if props["dbSchema"] and props["dbDesign"]:
  usage("Only one of the '-dbSchema' and '-dbDesign' parameters must be specified.")
  sys.exit(1)
#endif

if props["dbDesign"]:
  props["dbDesign"] = os.path.normpath(props["dbDesign"])
  if not os.path.isfile(props["dbDesign"]):
    usage("Database design file '" + props["dbDesign"] + "' does not exist.")
    sys.exit(1)
  #endif
#endif

# ------------------------------------------------------------
# validate the 'slice' value
intError = validateIntValue(props, "slice", MIN_SLICE, MAX_SLICE)
if intError:
  usage(intError)
  sys.exit(0)
#endif

# -----------------------------------------------
# check if the scope is a valid deployment target
scope = Scope(nodeName = props["node"], serverName = props["server"], clusterName = props["cluster"])
if scope.convertToCluster():
  print "Warning: '" + props["server"] + "' is a member of cluster '" + scope.clusterName + "', adjusting scope to cluster."
#endif

if not scope.isDeploymentTarget():
  usage(str(scope) + " is not a valid deployment target.")
  sys.exit(1)
#endif

if not scope.isBPCDeploymentTarget("RUNTIME"):
  usage("Process Choreographer is not configured on " + scope.displayName() +"." )
  sys.exit(1)
#endif

if scope.getCluster():
  foundMemberOnLocalNode = 0
  for member in wsadminToList(scope.list("ClusterMember")):
    if AdminConfig.showAttribute(member, "nodeName") == props["node"]:
      foundMemberOnLocalNode = 1
      break
    #endif
  #endfor
  if not foundMemberOnLocalNode:
    usage("No member of cluster " + props["cluster"] + " is defined on local node " + props["node"] + ".")
    sys.exit(1)
  #endif
#endif

# ---------------------------------------------------------------
# set options for data migration and invoke the DB migration tool
properties = addJDBCDriverForBPEDBToClasspath(scope, props)
options= HashMap()
options.put(DataMigrationMgr.SLICE_SIZE, Integer(props["slice"]))
if props["force"]:
  options.put(DataMigrationMgr.FORCE, "true")
if props["skipTablespaceMigration"]:
  options.put(DataMigrationMgr.SKIP_TABLESPACE, "true")
if props.has_key("BPETS8K"):
  options.put(DataMigrationMgr.BPETS8K, props["BPETS8K"])
try:
  DataMigrationMgr.start(props["dbUser"], props["dbPassword"], props["dbSchema"], props["className"], properties, options)
except:
  printException()

# ------------------------------------------------------------------------------
# check if custom tables have been configured and if so, print a warning message
# (see also http://www-1.ibm.com/support/docview.wss?uid=swg27010849&aid=1)
bfm = scope.list("BPELFlowManager")
if bfm == "":
  print "ERROR: The Process Choreographer configuration is corrupted - no 'BPELFlowManager' found in " + str(scope) + "."
#endif
props = wsadminToList(AdminConfig.showAttribute(bfm, "properties"))
for prop in props:
  if AdminConfig.showAttribute(prop, "name") == "customTableDefinition":
    value = AdminConfig.showAttribute(prop, "value")
    print "*******************************************************************************"
    print "NOTE: Custom property 'customTableDefinition' has been set to"
    print "  " + value
    value = os.path.normpath(AdminTask.expandBPCVariables(scope.asParameterList(["-variableString", value], 1)))
    if not os.path.isfile(value):
      print "which resolves to"
      print "  " + value
      print "but this file doesn't exists locally."
    #endif
    print "If you were using a custom table definition file for named materialized views,"
    print "they have been dropped by this data migration script. WebSphere Process Server"
    print "will only be able to recreate the materialized views during server startup"
    print "if the 'customTableDefinition' property points to a custom table definition"
    print "file that is accessible by WebSphere Process Server."
    print "If you were using a custom table definition file for custom tables, they must"
    print "be dropped and recreated manually now."
    print "*******************************************************************************"
    break
  #endif
#endfor

print __file__ + " finished."
