#
# Licensed Materials - Property of IBM
# 5655-FLW (C) Copyright IBM Corporation 2008, 2010. All Rights Reserved.
# US Government Users Restricted Rights- Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#

import os, re, traceback

from java.lang import String, System
from com.ibm.wsspi.scripting import ScriptingExtensionHelper
from com.ibm.bpe.config.util.db import BPCJDBCConfigUtils

__file__ = "bpclib.py"

#*******************************************************************************************************
# See http://www-01.ibm.com/support/docview.wss?rs=180&uid=swg21305344 for NLS considerations.
#*******************************************************************************************************

#*******************************************************************************************************
# class Scope
#*******************************************************************************************************
class Scope:

  def __init__(self):
    self.cellName = None
    self.clusterName = None
    self.currentNode = None
    self.nodeName = None
    self.serverName = None
  #enddef

  def __init__(self, **args):
    if args.has_key("cellName") and args["cellName"]:
      self.cellName = args["cellName"]
    elif wsadmin.getConnectionStatus() == WsAdmin.CONNECTED:
      self.cellName = AdminControl.getCell()
    else:
      self.cellName = System.getProperty("local.cell")
    #endif
    if args.has_key("nodeName") and args["nodeName"]:
      self.currentNode = args["nodeName"]
    elif wsadmin.getConnectionStatus() == WsAdmin.CONNECTED:
      self.currentNode = AdminControl.getNode()
    else:
      self.currentNode = System.getProperty("local.node")
    #endif
    if args.has_key("clusterName") and args["clusterName"]:
      self.clusterName = args["clusterName"]
      self.nodeName = None
      self.serverName = None
    else:
      self.clusterName = None
      self.nodeName = self.currentNode
      if args.has_key("serverName") and args["serverName"]:
        self.serverName = args["serverName"]
      else:
        self.serverName = None
      #endif
    #endif
  #enddef

  def __str__(self):
    return self.getContainmentPath()
  #enddef

  def getContainmentPath(self):
    path = "/Cell:" + self.cellName
    if self.clusterName:
      path = path + "/ServerCluster:" + self.clusterName
    else:
      if self.nodeName:
        path = path + "/Node:" + self.nodeName
        if self.serverName:
          path = path + "/Server:" + self.serverName
      #endif
    #endif
    return path + "/"
  #enddef

  def getConfigID(self):
    return AdminConfig.getid(self.getContainmentPath())
  #enddef

  def getObjectName(self):
    name = "WebSphere:cell=" + self.cellName
    if self.clusterName:
      name = name + ",cluster=" + self.clusterName
    else:
      if self.nodeName:
        name = name + ",node=" + self.nodeName
        if self.serverName:
          name = name + ",server=" + self.serverName
      #endif
    #endif
    return name
  #enddef

  def getCell(self):
    if self.cellName:
      return "/Cell:" + self.cellName + "/"
    return None
  #endif

  def getNode(self):
    cell = self.getCell()
    if cell and self.nodeName:
      return cell + "Node:" + self.nodeName + "/"
    return None
  #endif

  def getServer(self):
    node = self.getNode()
    if node and self.serverName:
      return node + "Server:" + self.serverName + "/"
    return None
  #endif

  def getCluster(self):
    cell = self.getCell()
    if cell and self.clusterName:
      return cell + "ServerCluster:" + self.clusterName + "/"
    return None
  #endif

  def convertToCluster(self):
    if self.getServer():
      server = self.getConfigID()
      if server:
        self.clusterName = AdminConfig.showAttribute(server, "clusterName")
        return self.getCluster()
      #endif
    #endif
    return 0
  #enddef

  def asParameterList(self, additionalParms = [], includeClusterNode = 0):
    parms = []
    if self.clusterName:
      parms.append("-clusterName")
      parms.append(self.clusterName)
      if includeClusterNode:
        parms.append("-nodeName")
        parms.append(self.currentNode)
    else:
      if self.nodeName:
        parms.append("-nodeName")
        parms.append(self.nodeName)
        if self.serverName:
          parms.append("-serverName")
          parms.append(self.serverName)
      #endif
    #endif
    if additionalParms:
      parms.extend(additionalParms)
    return parms
  #enddef

  def list(self, type):
    id = self.getConfigID()
    if id:
      return AdminConfig.list(type, id)
    return None
  #enddef

  def listApplications(self, pattern):
    pattern = re.compile(pattern)
    localApps = wsadminToList(AdminApp.list(self.getObjectName()))
    return [appName for appName in localApps if pattern.match(appName)]
  #enddef

  def isDeploymentTarget(self):
    return (self.getServer() or self.getCluster()) and self.getConfigID()
  #enddef

  def isBPCDeploymentTarget(self, operationMode = "RUNTIME"):
    if not self.isDeploymentTarget():
      return 0
    # AdminTask returns 'server1(cells/viennaNode01Cell/nodes/viennaNode01/servers/server1|server.xml)'
    # but getConfigID() returns 'server1(cells/viennaNode01Cell/nodes/viennaNode01/servers/server1|server.xml#Server_1183122130078)'
    id = String(self.getConfigID()).replaceFirst("#[^)]+", "")
    return id in wsadminToList(AdminTask.listBPCContainerDeploymentTargets(["-operationMode", operationMode]))
  #enddef

  def isBPCEventCollectorDeploymentTarget(self):
    if not self.isDeploymentTarget():
      return 0
    # AdminTask returns 'server1(cells/viennaNode01Cell/nodes/viennaNode01/servers/server1|server.xml)'
    # but getConfigID() returns 'server1(cells/viennaNode01Cell/nodes/viennaNode01/servers/server1|server.xml#Server_1183122130078)'
    id = String(self.getConfigID()).replaceFirst("#[^)]+", "")
    return id in wsadminToList(AdminTask.listBPCEventCollectorDeploymentTargets())
  #enddef

  def lookup(self, lookupScope, attribute):
    if not attribute or attribute == "":
      return None
    #endif
    object = None
    if not object and self.getServer():
      object = lookupScope(attribute, self.getServer())
    #endif
    if not object and self.getNode():
      object = lookupScope(attribute, self.getNode())
    #endif
    if not object and self.getCluster():
      for member in wsadminToList(self.list("ClusterMember")):
        if AdminConfig.showAttribute(member, "nodeName") == self.currentNode:
          nodeScope = Scope(cellName = self.cellName, nodeName = self.currentNode)
          object = lookupScope(attribute, nodeScope.getContainmentPath())
        #endif
      #endfor
      if not object:
        object = lookupScope(attribute, self.getCluster())
      #endif
    #endif
    if not object:
      object = lookupScope(attribute, self.getCell())
    #endif
    return object
  #enddef

  def displayName(self):
    if self.getServer():
      return "server " + self.serverName + " on node " + self.nodeName
    elif self.getCluster():
      return "cluster " + self.clusterName
    elif self.getNode():
      return "node " + self.nodeName
    #endif
    return None
  #enddef

  def lookupMBean(self, type, all = 0):
    mbeans = []
    if self.getServer():
      mbean = AdminControl.completeObjectName("type=" + type + ",node=" + self.nodeName + ",process=" + self.serverName + ",*")
      if mbean:
        mbeans.append(mbean)
    elif self.getCluster():
      cluster = self.getConfigID()
      for member in wsadminToList(self.list("ClusterMember")):
        nodeName = AdminConfig.showAttribute(member, "nodeName")
        serverName = AdminConfig.showAttribute(member, "memberName")
        mbean = AdminControl.completeObjectName("type=" + type + ",node=" + nodeName + ",process=" + serverName + ",*")
        if mbean:
          mbeans.append(mbean)
          if not all:
            break
        #endif
      #endfor
    #endif
    if mbeans and not all:
      return mbeans[0]
    return mbeans
  #enddef

#endclass

#*******************************************************************************************************
# class WsAdmin
#*******************************************************************************************************
def __showAttribute__(AdminConfig, object, attribute):
  if not object:
    raise AttributeError, "'None' object has no attribute " + str(attribute)
  #endif
  return AdminConfig.showAttribute0(object, attribute)
#enddef

class WsAdmin:

  __connectionStatus = "UNKNOWN"

  CONNECTED = "CONNECTED"
  DISCONNECTED = "DISCONNECTED"
  ERROR = "ERROR"

  def __init__(self):
    if not hasattr(AdminConfig, "showAttribute0"):
      AdminConfig.__class__.__dict__["showAttribute0"] = getattr(AdminConfig, "showAttribute")
      AdminConfig.__class__.__dict__["showAttribute"] = __showAttribute__
    #endif
  #enddef

  def getConnectionStatus (self):
    if WsAdmin.__connectionStatus != "UNKNOWN":
      return WsAdmin.__connectionStatus
    if not ScriptingExtensionHelper.getCurrentScriptingConfigService():
      WsAdmin.__connectionStatus = WsAdmin.ERROR
    elif not ScriptingExtensionHelper.getCurrentScriptingAdminClient():
      WsAdmin.__connectionStatus = WsAdmin.DISCONNECTED
    else:
      WsAdmin.__connectionStatus = WsAdmin.CONNECTED
    #endif
    return WsAdmin.__connectionStatus
  #enddef

  def parse(self, argv, params):
    options = ["-" + key for key in params.keys() if params[key] == None] # get list of options (keys for which the value is 'None')
    for key in [key for key in params.keys() if params[key] == ""]:
      params[key] = None # map empty string defaults to 'None' (compatibility)
    result = params.copy()
    lastArg = None
    for arg in argv:
      if arg.startswith("-"):
        if lastArg and not lastArg in options:
          raise ValueError, "Missing value for command line argument '" + lastArg + "'"
        lastArg = arg
        if not params.has_key(lastArg[1:]):
          raise ValueError, "Invalid command line argument '" + lastArg + "'"
        if lastArg in options:
          if result[lastArg[1:]] != None:
            raise ValueError, "Duplicate command line argument '" + lastArg + "'"
          result[lastArg[1:]] = 1
        #endif
      else:
        if not lastArg:
          raise ValueError, "Missing argument for command line value '" + arg + "'"
        if lastArg in options:
          raise ValueError, "Command line argument '" + lastArg + "' does not take a value - '" + arg + "'"
        if result[lastArg[1:]] != params[lastArg[1:]]:
          raise ValueError, "Duplicate command line argument '" + lastArg + "'"
        result[lastArg[1:]] = arg
        lastArg = None
      #endif
    #endfor
    if lastArg and not lastArg in options:
      raise ValueError, "Missing value for command line argument '" + lastArg + "'"
    return result
  #enddef

#endclass

#*******************************************************************************************************
# global scripting client
#*******************************************************************************************************
wsadmin = WsAdmin()

#*******************************************************************************************************
# print an exception
#*******************************************************************************************************
def printException():
  (etype, value, tb) = sys.exc_info()
  if not etype:
    return
  print "Exception received:"
  if not isinstance(value, java.lang.Throwable):
    print "".join(traceback.format_exception(etype, value, tb))
  else:
    prefix = ""
    while value:
      print prefix + str(value)
      value = value.getCause()
      prefix = "Caused by "
    #endwhile
  #endif
#enddef

#*******************************************************************************************************
# find a DataSource exactly in a give scope
#*******************************************************************************************************
def findDataSource(jndiName, scopePath):
  scope = AdminConfig.getid(scopePath)
  if not scope:
    return None
  for dataSource in AdminConfig.list("DataSource", scope).splitlines():
    if (jndiName == AdminConfig.showAttribute(dataSource, "jndiName")):
      # candidate, but may be in a sub-scope. construct id in desired scope to make sure.
      provider = AdminConfig.showAttribute(dataSource, "provider")
      providerName = AdminConfig.showAttribute(provider, "name")
      # Robustness: apparently, there may be multiple JDBC providers with the same name. In this case
      # the originally used 'AdminConfig.getid(scopePath + "JDBCProvider:" + providerName + "/DataSource:" + dataSourceName + "/")'
      # will return an invalid config id that causes problems upstream.
      for provider in AdminConfig.getid(scopePath + "JDBCProvider:" + providerName + "/").splitlines():
        for dataSource2 in AdminConfig.list("DataSource", provider).splitlines():
          if dataSource == dataSource2:
            break
        #endfor
      #endfor
      if dataSource:
        return dataSource
      #endif
    #endif
  #endfor
  return None
#enddef

#*******************************************************************************************************
# @EF rework: convert a string as returned by AdminConfig.show[Attribute] to a Jython list
#*******************************************************************************************************
def wsadminToList(inStr):
  outList=[]
  if (len(inStr)>0 and inStr[0]=='[' and inStr[-1]==']'):
    inStr = inStr[1:-1]
    tmpList = inStr.split(" ")
  else:
    tmpList = inStr.split("\n")  #splits for Windows or Linux
  for item in tmpList:
    item = item.rstrip();        #removes any Windows "\r"
    if (len(item)>0):
      outList.append(item)
  return outList
#endDef

#*******************************************************************************************************
# check if a given string is a valid namespace identifier
#*******************************************************************************************************
def isValidNameSpace(nameSpace):
  from java.lang import String
  NAMESPACE_REGEX = "(([a-zA-Z][0-9a-zA-Z+\\-\\.]*:)?/{0,2}[0-9a-zA-Z;/?:@&=+$\\.\\-_!~*'()%]+)?(#[0-9a-zA-Z;/?:@&=+$\\.\\-_!~*'()%]+)?"
  return String(nameSpace).matches(NAMESPACE_REGEX)
#enddef

#*******************************************************************************************************
# make sure a given time stamp is UTC and contains minutes, hours, and seconds
#*******************************************************************************************************
def normalizeTimeStamp(timeStamp, convertToUTC, mayOmitTime):

  from java.lang import String
  from java.text import SimpleDateFormat
  from java.util import TimeZone

  timeStamp = String(timeStamp)

  if mayOmitTime and timeStamp.matches("[-:0-9]+T\d{2}:\d{2}"):
    timeStamp = String(timeStamp.concat(":00"))

  try:
    if mayOmitTime and timeStamp.length() < 16:
      s = SimpleDateFormat("yyyy-MM-dd")
    else:
      s = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss")
    s.setLenient(0)
    utc = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss")
    if convertToUTC:
      utc.setTimeZone(TimeZone.getTimeZone("UTC"))
    timeStamp = utc.format(s.parse(timeStamp))
  except:
    timeStamp = None

  return timeStamp
#enddef

#*******************************************************************************************************
# validate integer parameter
#*******************************************************************************************************
def validateIntValue(props, key, minValue, maxValue):
  errorMsg = None
  try:
    props[key] = int(props[key])
  except ValueError:
    errorMsg = "Error: '" + key + "' must be an integer."
  else:
    if props[key] < minValue or props[key] > maxValue:
      errorMsg = "Error: '" + key + "' must be in the range from " + str(minValue) + " to " + str(maxValue) + "."
    #endif
  #endtry
  return errorMsg
#enddef

#*******************************************************************************************************
# extend the scripting client's classpath to include the JDBC driver for data source "jdbc/BPEDB"
#*******************************************************************************************************
def addJDBCDriverForBPEDBToClasspath(scope, props):

  from java.lang import Thread
  from java.util import HashMap
  from java.net import URL, URLClassLoader

  # find the BPC data source and query its provider's properties
  dataSource = scope.lookup(findDataSource, "jdbc/BPEDB")

  if not dataSource:
    usage("Data source 'jdbc/BPEDB' not found in scope " + str(scope))
    sys.exit(0)
  #endif

  provider = AdminConfig.showAttribute(dataSource, "provider")
  providerType = AdminConfig.showAttribute(provider, "providerType")
  classpath = AdminConfig.showAttribute(provider, "classpath")
  className = AdminConfig.showAttribute(provider, "implementationClassName")
  authAlias = AdminConfig.showAttribute(dataSource, "authDataAlias")

  if providerType != "Cloudscape JDBC Provider (XA)" and providerType != "Derby JDBC Provider (XA)" and providerType != "Derby JDBC Provider 40 (XA)" and not props["dbPassword"]:
    usage("The '-dbPassword' parameter is required for " + providerType + ".")
    sys.exit(0)
  #endif

  props["className"] = className

  authUser = None
  if authAlias:
    security = AdminConfig.getid(scope.getCell() + "Security:/")
    authDataEntries = wsadminToList(AdminConfig.showAttribute(security, "authDataEntries"))
    for authDataEntry in authDataEntries:
      if AdminConfig.showAttribute(authDataEntry, "alias") == authAlias:
        authUser = AdminConfig.showAttribute(authDataEntry, "userId")
        break
      #endif
    #endfor
  #endif

  # -------------------------------
  # read the DB design file, if any
  if props["dbDesign"]:
    dbprops = AdminTask.processBPCDatabaseScript(["-delayConfig", "true", "-noexec", "true", "-file", props["dbDesign"]])
    for key, value in [line.split("=", 1) for line in dbprops.splitlines()]:
      if key in ["dbSchema", "BPETS8K"]:
        props[key] = value
  #endif

  # ---------------------------------
  # compute the effective schema name
  if not props["dbUser"] and authUser:
    props["dbUser"] = authUser
  #endif

  if props["dbSchema"]:
    schema = props["dbSchema"]
    print "Using supplied schema qualifier '" + schema + "'."
  else:
    dataStore = scope.list("BPCDataStore")
    # WASX7017E: Exception received while running file "migrateDB.py"; exception information: com.ibm.ws.scripting.ScriptingException:
    # WASX7444E: Invalid parameter value "" for parameter "config id" on command "showAttribute"
    if dataStore:
      schema = AdminConfig.showAttribute(dataStore, "schemaName")
    else:
      print "Warning: BPCDataStore not found in " + str(scope) + ", assuming no schema qualifier has been configured."
      print "This can happen if migrating from WPS 6.0.2 without running the BPMMigrateCluster script."
      schema = None
    #endif
    if schema:
      print "Using configured schema qualifier '" + schema + "'."
    elif authUser:
      schema = authUser
      print "Using implicit schema qualifier '" + schema + "'."
    #endif
    if schema:
      props["dbSchema"] = schema
    #endif
  #endif

  # --------------------------------------------------------
  # prepare a java.util.Map of data source custom properties
  resourceProperties = AdminConfig.showAttribute(AdminConfig.showAttribute(dataSource, "propertySet"), "resourceProperties")
  properties = HashMap()
  url = None
  for property in wsadminToList(resourceProperties):
    name = AdminConfig.showAttribute(property, "name")
    value = AdminConfig.showAttribute(property, "value")
    if (providerType.startswith("Derby") or providerType.startswith("Cloudscape")) and name == "databaseName":
      # the migrated provider still has providerType="Cloudscape JDBC Provider (XA)" although it is based on Derby now
      value = os.path.normpath(AdminTask.expandBPCVariables(scope.asParameterList(["-variableString", value], 1)))
    #endif
    properties.put(name, value)
  #endfor

  # CommonDB only sets URL, fill in driverType, serverName, portNumber, and databaseName if these are not set
  if providerType.startswith("Oracle"):
    BPCJDBCConfigUtils.parseURLProperties(properties)
  #endif

  # -------------------------------------------------------------
  # set up classpath for JDBC provider, based on settings in WCCM
  classpath = AdminTask.expandBPCVariables(scope.asParameterList(["-variableString", classpath], 1))
  urls = []
  for path in classpath.split(";"):
    path = os.path.normpath(path)
    if not os.path.exists(path):
      print "File " + path + " does not exist, ignoring."
    else:
      url = URL("file", "", -1, path)
      print "File " + path + " exists, adding it to the classpath."
      urls.append(url)
    #endif
  #endfor

  classloader = URLClassLoader(urls, Thread.currentThread().getContextClassLoader())
  Thread.currentThread().setContextClassLoader(classloader)

  return properties
#enddef

