################################################################################
# BEGIN COPYRIGHT
################################################################################
#
# THIS PRODUCT CONTAINS RESTRICTED MATERIALS OF IBM
# All Rights Reserved * Licensed Materials - Property of IBM
# 5655-FLW (C) Copyright IBM Corporation 2005, 2007
# US Government Users Restricted Rights - Use, duplication or disclosure
# restricted by GSA ADP Schedule Contract with IBM Corp.
#
################################################################################
# END COPYRIGHT
################################################################################
#
# Jython module containing a profile implementation for the Observer
#
# Version: %I% %E% %U%
#
################################################################################
__file__="bpcoProfile.py"


################################################################################
#
# imported jython/python modules
#
################################################################################
import sys
import time
import jarray

import bpelib
import os.path
import java.lang.System

#BP602 import zipfile
from   java.util.jar import *
from   java.util     import Properties
from   java.io       import FileInputStream, FileOutputStream


#some shortcuts:
log  = bpelib.log
mget = bpelib.mcat.get

sysEnv=java.lang.System.getProperties()
lineSep=sysEnv["line.separator"]

################################################################################
#
# ConfigurationParameter
#
################################################################################
class ConfigurationParameter:


  STRINGTYPE  = "String"
  INTEGERTYPE = "Integer"
  BOOLEANTYPE = "Boolean"

  #----------------------------------------------------------------------------
  # Ctor
  #----------------------------------------------------------------------------
  def __init__(self, key, dtype, value=None, adminTaskParameter=None, unit=None,description=None):

    self.key  = key
    self.value= value
    self.dtype= dtype
    self.unit = unit
    self.description=description
    self.adminTaskParameter=adminTaskParameter

  #----------------------------------------------------------------------------
  # check if the value passed is valid for this type
  #----------------------------------------------------------------------------
  def isValid(self, value):
    #check if the value is valid for this type
    return 1



################################################################################
#
# IntegerParameter
#
################################################################################
class IntegerParameter(ConfigurationParameter):

  #----------------------------------------------------------------------------
  # Ctor
  #----------------------------------------------------------------------------
  def __init__(self, key, value=0, adminTaskParameter=None, unit=None,description=None):

    ConfigurationParameter.__init__( self,
                                     key,
                                     ConfigurationParameter.INTEGERTYPE,
                                     value,
                                     adminTaskParameter,
                                     unit,
                                     description )

  #----------------------------------------------------------------------------
  # check if the value passed is valid for this type
  #----------------------------------------------------------------------------
  def isValid(self,value):
    try:
      int(value)
    except:
      return 0

    return 1  # == true, is a valid int


################################################################################
#
# StringParameter
#
################################################################################
class StringParameter(ConfigurationParameter):

  #----------------------------------------------------------------------------
  # store the file
  #----------------------------------------------------------------------------
  def __init__(self, key, value=None, adminTaskParameter=None, unit=None,description=None):

    ConfigurationParameter.__init__( self,
                                     key,
                                     ConfigurationParameter.STRINGTYPE,
                                     value,
                                     adminTaskParameter,
                                     unit,
                                     description )

################################################################################
#
# BooleanParameter
#
################################################################################
class BooleanParameter(StringParameter):

  TRUE  = "true"
  FALSE = "false"

  #----------------------------------------------------------------------------
  # Ctor
  #----------------------------------------------------------------------------
  def __init__(self, key, value="true", adminTaskParameter=None, unit=None,description=None):

    if self.isValid(value):
      ConfigurationParameter.__init__( self,
                                       key,
                                       ConfigurationParameter.BOOLEANTYPE,
                                       value,
                                       adminTaskParameter,
                                       unit,
                                       description )
    else:
      raise ValueError, mget("ErrorInvalidValueForKey")

  #----------------------------------------------------------------------------
  # check if the value passed is valid for this type
  #----------------------------------------------------------------------------
  def isValid(self,value):

    rc=0

    if (value==BooleanParameter.FALSE) or (value==BooleanParameter.TRUE) :
      rc=1   # == true, is a valid int

    return rc

################################################################################
#
# ConfigurationProfile
#
#
################################################################################
class ConfigurationProfile:


  knownKeys = {}

  #init the knownKeys

  #
  # Common keys
  #
  knownKeys["observerSchemaName"]=StringParameter(
    key  = "observerSchemaName",
    adminTaskParameter="-observerSchemaName",
    description = mget("ProfileDescObsSchema"))

  knownKeys["observerCreateTables"]=BooleanParameter(
    key  = "observerCreateTables",
    value = BooleanParameter.FALSE,
    adminTaskParameter="-observerCreateTables",
    description = mget("ProfileDescObsCreaSchema"))


  #
  #  Observer properties
  #
  knownKeys["ReportAtSnapshotRange"]=IntegerParameter(
    key   = "ReportAtSnapshotRange",
    unit  = "days",
    adminTaskParameter="-reportAtSnapshotRange",
    description=mget("ProfileDescObsReportRange") )

  #
  #  EventCollector properties
  #

  knownKeys["BPCEventTransformerEventCount"]=IntegerParameter(
    key   = "BPCEventTransformerEventCount",
    unit  = "Events",
    adminTaskParameter="-bpcEventTransformerEventCount",
    description=mget("ProfileDescTransECount") )

  knownKeys["BPCEventTransformerMaxWaitTime"]=IntegerParameter(
    key   = "BPCEventTransformerMaxWaitTime",
    unit  = "Minutes",
    adminTaskParameter="-bpcEventTransformerMaxWaitTime",
    description=mget("ProfileDescTransMaxWait") )

  knownKeys["BPCEventTransformerToleranceTime"]=IntegerParameter(
    key   = "BPCEventTransformerToleranceTime",
    unit  = "Minutes",
    adminTaskParameter="-bpcEventTransformerToleranceTime",
    description=mget("ProfileDescTransTolTime") )



  #----------------------------------------------------------------------------
  # Ctor
  #----------------------------------------------------------------------------
  def __init__(self,fileName, console = None):

    self.propertyFile=fileName
    self.keys = {}
    self.changed = 0        # keep track of edit changes
    self.manifest = None    # member to keep the Manifest from older jars.

    if console == None:
      self.console = bpelib.ConsoleHandler()
    else:
      self.console = console

  #----------------------------------------------------------------------------
  # readFromJar, load the propertiesFile from within a jar/zip file
  #----------------------------------------------------------------------------
  def readFromJar(self, jarFileName, propertiesFileName=None):

    if propertiesFileName==None:
      propertiesFileName=self.propertyFile

    jarFile=JarFile(jarFileName)

    #get the properties File
    propertiesFileEntry=jarFile.getJarEntry(propertiesFileName)
    self.load(jarFile.getInputStream(propertiesFileEntry))

    #save the manifest
    manifest=jarFile.getManifest()
    if manifest != "" :
      self.manifest=manifest

    jarFile.close()

  #----------------------------------------------------------------------------
  # readFromJar, load the propertiesFile from within a jar/zip file
  #----------------------------------------------------------------------------
  def readFromEar(self,
                  earFileName,
                  configJarName="configuration.jar",
                  propertiesFileName=None):

    if propertiesFileName==None:
      propertiesFileName=self.propertyFile


    earFile=java.util.zip.ZipFile(earFileName)
    configJarEntry=earFile.getEntry(configJarName)

    tmpFile=os.path.join(sysEnv["user.install.root"],"temp")
    tmpFile=os.path.join(tmpFile, os.path.basename(earFileName) + "_" + configJarName)

    os  = FileOutputStream(tmpFile)
    ins = earFile.getInputStream(configJarEntry)
    buffer=jarray.zeros(configJarEntry.getSize(),"b")

    ins.read(buffer)
    os.write(buffer)

    ins.close()
    os.close()
    earFile.close()

    self.readFromJar( tmpFile )
    os.remove(tmpFile)

  #----------------------------------------------------------------------------
  # writeToJar, write the propertiesFile into a jar/zip file
  # if file exists, append.
  #----------------------------------------------------------------------------
  def writeToJar(self, jarFileName, propertiesFileName=None):

    if propertiesFileName==None:
      propertiesFileName=self.propertyFile

    if self.manifest!=None:
      jarFile=JarOutputStream( FileOutputStream(jarFileName), self.manifest )
    else:
      jarFile=JarOutputStream( FileOutputStream(jarFileName) )

    entry=JarEntry(propertiesFileName)
    jarFile.putNextEntry(entry)
    jarFile.write(self.asString(), 0 , len(self.asString()))

    jarFile.close()



#     jarfile=zipfile.ZipFile(jarFileName,"a")
#     propertiesFile=zipfile.ZipInfo(propertiesFileName,time.localtime())
#     jarfile.writestr(propertiesFile,self.asString())
#     jarfile.close()
  #----------------------------------------------------------------------------
  # load the file
  #----------------------------------------------------------------------------
  def loadFromStream(self, inStream=None):

    try:

      if inStream==None:
        fileInStream = FileInputStream(self.propertyFile)
      else:
        fileInStream=inStream

      p=Properties()
      p.load(fileInStream)
      names=p.propertyNames()

      while names.hasMoreElements():

        nextProp=names.nextElement()

        if not self.knownKeys.has_key(nextProp):

          log( mget("WarningUnknownParam", nextProp) )
          # add temporarily as known value
          self.knownKeys[nextProp]=StringParameter(nextProp,p[nextProp],None)
          # add refernce to the list of keys of the profile
          self.keys[nextProp]=self.knownKeys[nextProp]

        else:

          # get the object of the param
          param=self.knownKeys[nextProp]

          #check if the value in the file is valid for this type
          if param.isValid( p[nextProp] ):
            param.value=p[nextProp]
            self.keys[nextProp]=self.knownKeys[nextProp]
          else:
            log( mget("ErrorInvalidValueForKey", nextProp ) )
            log( mget("UsingDefault", str(param.value)) )

    finally:
      fileInStream.close()




  #----------------------------------------------------------------------------
  # load the file
  #----------------------------------------------------------------------------
  def loadFromString(self, inString):

    lines=inString.split(lineSep)

    for line in lines:

      if line=="":
        continue

      if line.find("=") == -1:
        continue

      key=line.split("=",1)[0]
      val=line.split("=",1)[1]

      if not self.knownKeys.has_key(key):

        log( mget("WarningUnknownParam", key) )
        # add temporarily as known value
        # self.knownKeys[key]=StringParameter(key,val,None)
        # add refernce to the list of keys of the profile
        # self.keys[key]=self.knownKeys[key]

      else:

        # get the object of the param
        param=self.knownKeys[key]

        #check if the value in the file is valid for this type
        if param.isValid( val ):
          param.value=val
          self.keys[key]=self.knownKeys[key]
        else:
          log( mget("ErrorInvalidValueForKey", key ) )
          log( mget("UsingDefault", str(param.value)) )


  #----------------------------------------------------------------------------
  # store the file
  #----------------------------------------------------------------------------
  def save(self):


      fileOutStream = FileOutputStream(self.propertyFile)
      p=Properties()

      #map directory to PropFile
      for key in self.getKeyList():
        p[key] = str(self.keys[key].value)

      p.store(fileOutStream,mget("ProfileHeader"))

      fileOutStream.close()
      self.changed = 0


  #----------------------------------------------------------------------------
  # get a jython list of the property names
  #----------------------------------------------------------------------------
  def getKeyList(self):

    keylist=self.keys.keys()
    keylist.sort()
    return keylist

  #----------------------------------------------------------------------------
  # get a jython list of the property names
  #----------------------------------------------------------------------------
  def asString(self):

    out="#" + mget("ProfileHeader") + lineSep
    out=out + time.strftime("#%a %b %d %X %Z %Y") + lineSep   #Tue Feb 28 18:59:25 CET 2006

    for key in self.getKeyList():
      out=out+ key + "=" + str(self.keys[key].value) + lineSep

    return out

  #----------------------------------------------------------------------------
  #
  # editProfile
  #
  # Provide a menu to edit values base on the type defined in the knownKeys
  # dictionary
  #----------------------------------------------------------------------------
  def editProfile(self):

    indent=3


    while 1:

      print mget("NewLine")
      print mget("EditProfileMenuTitle" )
      print mget("NewLine")

      keys=self.getKeyList()
      for index in range(0,len(keys)):
        key=keys[index]
        print str(index+1).rjust(indent)+")", key[0].upper()+key[1:]

      print mget("NewLine")
      print "  0)", mget("Menu_Exit")

      print mget("NewLine")
      ans = raw_input( mget("YourSelection") + ' ' )
      print mget("NewLine")

      if "x" in ans or "0" in ans:
        break

      if ans=="":
        continue

      try:
        ans=int(ans)
      except :
        print mget("Menu_InvalidSelection", str(ans) )
        print mget("NewLine")
        continue

      if ans>index+1 or ans<1:
        print mget("Menu_InvalidSelection", str(ans) )
        print mget("NewLine")
        continue

      self.editValue(keys[ans-1])

    return

  #----------------------------------------------------------------------------
  # get a property/config value,
  #----------------------------------------------------------------------------
  def getValue(self,key):

    return self.keys[key].value

  #----------------------------------------------------------------------------
  # get a property/config value,
  #----------------------------------------------------------------------------
  def setValue(self,key,value):

    p=self.keys[key]

    if p.isValid(value)==1:
      if value!=p.value:
        # we got a profile change
        self.changed = 1
        p.value=value
        return 0
    else:
      log( mget("ErrorInvalidValueForKey",key, value) )
      return -1

  #----------------------------------------------------------------------------
  # edit a property/config value,
  # -> provide a prompt an some checkings
  #----------------------------------------------------------------------------
  def editValue(self,key):

    indent="  "
    tabStop=len(indent + mget("ConfigurationParameter") )
    rightMargin=79
    param=self.keys[key]

    while 1:

      print mget("NewLine")
      print mget("EditConfigurationParam")
      print mget("NewLine")
      print indent + mget("ConfigurationParameter").ljust(tabStop) + param.key[0].upper() + param.key[1:]

      if param.description!=None:
        outStr=self.console.formatWithMargins( param.description, tabStop+1, rightMargin)
        print indent + mget("Description").ljust(tabStop) + outStr[0].strip()

        for line in range(1,len(outStr)):
          print outStr[line],

        print


      print indent + mget("DataType").ljust(tabStop) + param.dtype
      if param.unit!=None:
        print indent + mget("Unit").ljust(tabStop) + param.unit

      value=str(param.value)
      if value=="" or value=="None":
        value=mget("ValueNotSet")

      print indent + mget("CurrentValue").ljust(tabStop)+ "'" + value + "'"
      print mget("NewLine")

      print indent + mget("EnterNewValue") ,
      if param.dtype==ConfigurationParameter.STRINGTYPE:
        print mget("EnterSpaceToUnset"),
      print mget("EnterEnterToExit")
      result=raw_input(indent + mget("YourSelection", "") + " ")
      print mget("NewLine")

      if result=="":
        break

      if param.isValid(result) == 1:
        self.changed = 1
        param.value=result.strip()
        value=str(param.value)
        if value=="" or value=="None":
          value=mget("ValueNotSet")
        print "==>",  mget("ValueForKeySetTo", param.key, value)
        print mget("NewLine")
        break

      print ( mget("ErrorInvalidValueForKey",param.key, result) )
      print mget("NewLine")

    return



